.cascade-colorpicker .a-color-picker label {
  width: initial;
}
.cascade-colorpicker input[type="color"] {
  padding: 0;
  margin-left: -160px;
  margin-right: 130px;
  width: 30px;
  height: 30px;
  border: none;
}
.cascade-colorpicker input[type="text"] {
  text-align: right;
}
.cascade-colorpicker input[type="text"].disabled {
  color: #888;
  border-style: dashed;
}

.cascade-color-picker em {
  margin-left: 1em;
  margin-right: 1em;
}
.cascade-color-picker .a-color-picker {
  position: absolute;
  margin-top: -20px;
  margin-left: -60px;
  z-index: 1;
}
