{% load i18n %}{% spaceless %}
<div class="form-row">
{% for group, options, index in widget.optgroups %}{% with widget=options.0 %}
	<div class="fieldBox">
		<div class="panel {{ widget.value }}">
			<div class="panel-heading">{{ widget.label }}</div>
			<div class="panel-body">{% trans "Content" %}</div>
		</div>
		<div class="label">{% include widget.template_name %}</div>
	</div>
{% endwith %}{% endfor %}</div>
{% endspaceless %}