.preview-iconfont h2 {
	margin: 16px 0 8px;
}
.preview-iconfont ul {
	line-height: 42px;
}
.preview-iconfont ul li {
	box-sizing: border-box;
	display: inline;
	font-size: 28px;
	height: 48px;
	margin: 0 8px 8px 0;
	border: 2px solid transparent;
	border-radius: 50%;
}
.preview-iconfont ul li:hover {
	border-color: #880000;
}
