from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cmsplugin_cascade', '0006_bootstrapgallerypluginmodel'),
    ]

    operations = [
    ]
